import os
import shelve
from functools import wraps

_cachepath = os.path.join(os.path.dirname(__file__), 'snnm.cache')


def cached(f):
    @wraps(f)
    def wrapper(term):
        result = None
        cache = shelve.open(_cachepath)
        if term in cache:
            result = cache[term]
        else:
            result = f(term)
            cache.clear()
            cache[term] = result
        cache.close()
        return result
    return wrapper



import argparse
import sys

from .scrape import scrape
from .style import adorn


def parse(args):
    parser = argparse.ArgumentParser(
        prog='snnm',
        description='Get a list of synonyms from http://www.thesaurus.com'
    )
    parser.add_argument(
        'term',
        metavar='TERM',
        help='a string TERM representing the lookup term'
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-m',
        '--mixed-case',
        action='store_const',
        help='display the synonyms in mixedCaseStyle',
        dest='style',
        const='mixed'
    )
    group.add_argument(
        '-c',
        '--camel-case',
        action='store_const',
        help='display the synonyms in CamelCaseStyle',
        dest='style',
        const='camel'
    )
    group.add_argument(
        '-u',
        '--underscore',
        action='store_const',
        help='display the synonyms in lower case with underscore_style',
        dest='style',
        const='underscore'
    )
    group.add_argument(
        '-o',
        '--constant',
        action='store_const',
        help='display the synonyms in CONSTANT_STYLE',
        dest='style',
        const='constant'
    )
    return parser.parse_args(args)


def main():
    args = parse(sys.argv[1:])
    synonyms = scrape(args.term)
    synonyms = adorn(synonyms, args.style)
    for s in synonyms:
        print(s)



import re
from urllib.request import urlopen
from urllib.parse import quote
from urllib.error import HTTPError
import logging

from bs4 import BeautifulSoup

from .misc import cached

logger = logging.getLogger(__name__)
_baseurl = 'http://www.thesaurus.com/browse/'
_regex = re.compile(r'^[A-Za-z ]+$')


def _audit(term):
    logger.debug('Checking if term "%s" is valid' % term)
    if not re.match(_regex, term):
        logger.error(('Invalid term. Only letters and white spaces are '
                      'accepted' % term))
        logger.debug('Regex: ' + _regex.pattern)
        raise ValueError('"%s" is not a valid term')
    logger.debug('Valid term!')


def _fetch(url):
    logger.debug('Attempting to fetch the url: "%s"' % url)
    html = ''
    try:
        with urlopen(url) as r:
            html = str(r.read(), encoding='utf-8')
        logger.debug('Fetch succeeded!')
    except HTTPError:
        logger.error("-- NO MATCHES --")
    return html


def _parse(html):
    logger.debug('Attempting to parse HTML: %s (...)' % html[:250])
    synonyms = []
    soup = BeautifulSoup(html, 'lxml')
    divs = soup.find_all('div', class_='relevancy-list')
    logger.debug('divs found: %d' % len(divs))
    for d in divs:
        spans = d.find_all('span', class_='text')
        logger.debug('spans found: %s' % len(spans))
        synonyms += [str(s.string) for s in spans]
    logger.debug('Parse succeeded!')
    return synonyms


@cached
def scrape(term):
    logger.debug(('Attempting to scrape "%s" synonyms from '
                  'http://www.thesaurus.com'))
    _audit(term)
    html = _fetch(_baseurl + quote(term))
    synonyms = _parse(html)
    logger.debug('Scrape succeeded! Found %d synonyms: %s' % (len(synonyms),
                                                              synonyms))
    return synonyms



import re
import unicodedata

import logging
logger = logging.getLogger(__name__)


def _camel(sentence):
    rv = ''
    words = sentence.split()
    for w in words:
        rv += w.title()
    return rv


def _mixed(sentence):
    words = sentence.split()
    rv = words[0].lower()
    if len(words) > 1:
        rv += _camel(' '.join(words[1:]))
    return rv


def _underscore(sentence):
    rv = sentence.lower()
    rv = rv.replace(' ', '_')
    return rv


def _constant(sentence):
    return _underscore(sentence).upper()


_action = {
    'mixed': _mixed,
    'camel': _camel,
    'underscore': _underscore,
    'constant': _constant,
}


def _normalize(word):
    w = word.strip()
    w = re.sub(r'\s+', ' ', w)
    w = re.sub(r'[^\d\w\s]+', '', w)
    w = unicodedata.normalize('NFKD', w)
    w = w.encode('ascii', 'ignore').decode('utf-8')
    return w


def adorn(words, style):
    if style:
        logger.debug('Stylizing synonyms in a %s fashion' % style)
        adorned = []
        for w in words:
            nw = _normalize(w)
            adorned.append(_action[style](nw))
        return adorned
    return words



from setuptools import setup

setup(name='snnm',
      version='0.1.2',
      author='Gabriel Lima',
      author_email='ewilazarus@gmail.com',
      description='Naming aid',
      license='MIT',
      keywords='synonym',
      url='https://github.com/ewilazarus/snnm',
      packages=['snnm'],
      install_requires=['beautifulsoup4',],
      download_url='https://github.com/ewilazarus/snnm/tarball/0.1.2',
      entry_points={
          'console_scripts': [
              'snnm = snnm.cli:main'
          ]
      })
